<?php

declare(strict_types=1);

namespace FiloBlu\Refilo\Helper;

use Magento\Framework\ObjectManagerInterface;
use Magento\PageBuilder\Model\Dom\HtmlDocumentFactory;
use Pelago\Emogrifier\CssInliner;
use Symfony\Component\CssSelector\Exception\ParseException;
use function class_exists;

/**
 *
 */
class PageBuilderConverter
{

    /**
     * @var HtmlDocumentFactory | null
     */
    private $documentFactory;

    /**
     * @param ObjectManagerInterface $objectManager
     */
    public function __construct(
        ObjectManagerInterface $objectManager
    )
    {
        if (class_exists(HtmlDocumentFactory::class)) {
            $this->documentFactory = $objectManager->get(HtmlDocumentFactory::class);
        }
    }

    /**
     * @param $data
     * @return mixed|string
     * @throws ParseException
     */
    public function convert($data)
    {
        if (!$this->documentFactory && !method_exists(\Magento\PageBuilder\Model\Dom\Element::class, 'getOriginalElement')) {
            return $data;
        }
        $document = $this->documentFactory->create(['document' => $data]);
        $document->querySelector('body')->setAttribute('id', 'html-body');
        try {
            $css = $document->querySelector('style');
        } catch (\Throwable $e) {
            return $data;
        }

        $cssElement = '';
        if ($css) {
            $cssElement = $css->getOriginalElement()->nodeValue;
        }

        return CssInliner::fromHtml($document->getContents())->inlineCss($cssElement)->renderBodyContent();
    }

}
